package edu.neu.ccs.beans.demo.colorbeans;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.beans.beancontext.*;


/**
 * Class ColorBean acts as a Bean that can be added or removed from the
 * ColorBeanContext. It will notify its PropertyChangeListener(s) whenever
 * its color changes.
 */
public class ColorBean extends Canvas {

	private static final Dimension prefferedSize = new Dimension(75,45);
	private static final Dimension minimumSize = new Dimension(50,30);
	private static Color background = Color.red;
	private static Color foreground = Color.black;

	private PropertyChangeSupport colorChanges = new PropertyChangeSupport(this);
	/**
	 * Constructor
	 */
	public ColorBean() {
	    setSize(prefferedSize);
	    setForeground(foreground);
	    setBackground(background);
	}
	/**
	 * This method is used to add a PropertyChangeListener into the
	 * internal listener list.
	 * @param       listener    a PropertyChangeListener
	 */
	public void addColorListener(PropertyChangeListener listener) {
		colorChanges.addPropertyChangeListener(listener);
	}
	/**
	 * Color property represents the background color.
	 * @return Color  Color property
	 */
	public Color getColor() {
		return getBackground();
	}
	public Dimension getMinimumSize() {
		return minimumSize;
	}
	public Dimension getPrefferedSize() {
		return prefferedSize;
	}
	/** 
	 * Overide the paint method. Writes the name of the ColorBean.
	 */
	public void paint(Graphics g) {
		g.drawString(this.getName(), 5, (getFont().getSize() + 4));
	}
	/**
	 * This method is used to remove a PropertyChangeListener from the
	 * internal listener list.
	 * @param listener  a PropertyChangeListener
	 */
	public void removeColorListener(PropertyChangeListener listener) {
		colorChanges.removePropertyChangeListener(listener);
	}
	/**
	 * This method is used to set bean's color.
	 * Everytime the bean's color is changed, it will also fire the
	 * PropertyChangeEvent to notify all its PropertyChangeChangeListener(s)
	 * its color is changed.
	 * @param  newColor a Color object
	 */
	public void setColor(Color newColor) {
		Color oldColor = getBackground();
		setBackground(newColor);
		colorChanges.firePropertyChange(new PropertyChangeEvent(this, "color", oldColor, newColor));
		firePropertyChange("color", oldColor, newColor);
	}
}
